/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "AxisRecorder.h"
#include "controllers/common/family/Families.h"
#include "ics/InstrumentManager/InstrumentManager.h"
#include "controllers/common/axis/AxisController.h"
#include "controllers/lss/figaro/DetA.h"
#include "controllers/lss/figaro/DetH.h"

namespace axis {

const string AxisRecorder::TYPE = "axis_recorder";

AxisRecorder::AxisRecorder(const string& name) :
	ExperimentController(name), m_initialDate(common::Date().getTimeUs()) {

	setFamily(family::HIDDEN);

	string outputFilePath = getenv("HOME");
	outputFilePath += "/serverlog/axis.log";

	m_dataFile.reset(new ofstream(outputFilePath.c_str()));
}

AxisRecorder::AxisRecorder(const AxisRecorder& controller) :
	ExperimentController(controller) {

}

AxisRecorder::~AxisRecorder() {
}

void AxisRecorder::postConfiguration() {

	m_experimentControllersMap = InstrumentManager::getInstance()->getAllInstalledExperimentControllersMap();
	map<string, ExperimentController*>::iterator it;
	for (it = m_experimentControllersMap.begin(); it != m_experimentControllersMap.end(); it++) {

		// AxisController
		if (dynamic_cast<axis::AxisController*>(it->second) != 0) {
			axis::AxisController* controller = dynamic_cast<axis::AxisController*>(it->second);
			controller->attach(this);

			registerUpdater(controller->position, &AxisRecorder::updateActualPosition, this, controller->getName());
		}
		// DetA
		else if (dynamic_cast<figaro::DetA*>(it->second) != 0) {
			figaro::DetA* controller = dynamic_cast<figaro::DetA*>(it->second);
			controller->attach(this);

			registerUpdater(controller->angle, &AxisRecorder::updateActualPosition, this, controller->getName());
		}
		// DetH
		else if (dynamic_cast<figaro::DetH*>(it->second) != 0) {
			figaro::DetH* controller = dynamic_cast<figaro::DetH*>(it->second);
			controller->attach(this);

			registerUpdater(controller->height, &AxisRecorder::updateActualPosition, this, controller->getName());
		}
	}
}

void AxisRecorder::updateActualPosition(const std::string& controllerName) {
	common::Date now;

	if (*m_dataFile) {
		// AxisController
		if (dynamic_cast<axis::AxisController*>(m_experimentControllersMap[controllerName]) != 0) {
			axis::AxisController* controller = dynamic_cast<axis::AxisController*>(m_experimentControllersMap[controllerName]);
			(*m_dataFile) << (now.getTimeUs() - m_initialDate) << ", " << controllerName << ", " << controller->position() << endl;
		}
		// DetA
		else if (dynamic_cast<figaro::DetA*>(m_experimentControllersMap[controllerName]) != 0) {
			figaro::DetA* controller = dynamic_cast<figaro::DetA*>(m_experimentControllersMap[controllerName]);
			(*m_dataFile) << (now.getTimeUs() - m_initialDate) << ", " << controllerName << ", " << controller->angle() << endl;
		}
		// DetH
		else if (dynamic_cast<figaro::DetH*>(m_experimentControllersMap[controllerName]) != 0) {
			figaro::DetH* controller = dynamic_cast<figaro::DetH*>(m_experimentControllersMap[controllerName]);
			(*m_dataFile) << (now.getTimeUs() - m_initialDate) << ", " << controllerName << ", " << controller->height() << endl;

		}
	}
}

}

