/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "ListModeBlock.h"
#include "InstrumentAbstraction/AbstractController.h"
#include <common/base/ServerProperties.h>
#include <lstdpp128/lstdpp.h>
#include <boost/format.hpp>

using namespace boost;
using namespace lstdpp128;

namespace npp {

ListModeBlock::ListModeBlock(uint32 nbdetectors, const vector<AbstractController*>& detectorControllers) :
		DataBlock(true), m_NbDetectors(nbdetectors), m_DetectorControllers(
				detectorControllers), m_Oldlistmodedatasize(0) {
	// Empty
}

ListModeBlock::~ListModeBlock() {
	// Empty
}

void ListModeBlock::writeBlock(std::ofstream & file) {

	int32 *listmodedata = 0;
	uint32 listmodedatasize = 0;
	uint32 totallistmodedatasize = 0;

	ostringstream filename;
	filename << common::ServerProperties::getInstance()->getNomadDataPath() << boost::format("%06i") % m_RecordNumber << ".lst";
	ofstream dataFile;
	// Open data file
	dataFile.open(filename.str().c_str(), std::ios::app | std::ios::binary);

	if (dataFile) {

		// Verify if first write
		if (dataFile.tellp() == 0) {

			// Call to library with current context.
			string buffer = writeListModeContextToBinary(listModeContext);

			dataFile.write(buffer.c_str(), buffer.size());

			m_Oldlistmodedatasize = 0;
		}

		for (uint32 i = 0; i < m_NbDetectors; ++i) {
			// Write list mode
			if (m_DetectorControllers[i] != NULL) {
				listmodedata = any_cast<int32*>(m_DetectorControllers[i]->getValue("list_mode_data"));
				listmodedatasize = any_cast<int32>(m_DetectorControllers[i]->getValue("list_mode_data_size"));
				if ((listmodedatasize != 0) && (listmodedata != NULL)) {
					totallistmodedatasize += listmodedatasize / 4;
					dataFile.write((const char *) listmodedata, listmodedatasize * sizeof(int32));
					m_DetectorControllers[i]->setValue("list_mode_data_size", (int32) 0);
				}
			}
		}

		dataFile.close();
	}

	fstream dataFile1;
	dataFile1.open(filename.str().c_str(), ios_base::in | ios_base::out | std::ios::binary);
	if (dataFile1) {
		dataFile1.seekp(2 * sizeof(uint32), ios_base::beg);
		totallistmodedatasize += m_Oldlistmodedatasize;
		dataFile1.write((const char *) &totallistmodedatasize, sizeof(uint32));
		m_Oldlistmodedatasize = totallistmodedatasize;
		dataFile1.close();
	}

}

}
