/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef NPP_REMOTEDPPCOINCIDENCE_H
#define NPP_REMOTEDPPCOINCIDENCE_H

#include "RemoteDPPLiveProcess.h"
#include <boost/thread.hpp>
#include <stdint.h>
#include <memory>

namespace npp {

class RemoteDPPCoincidence : public RemoteDPPLiveProcess {

public:
	//! Type of controller
	static const std::string TYPE;

	RemoteDPPCoincidence(const std::string& name);
	RemoteDPPCoincidence(const RemoteDPPCoincidence& controller);
	virtual ~RemoteDPPCoincidence();

	virtual void postConfiguration();
	virtual void setDetectorControllers(const std::vector<ControllerPtr<acquisition::DetectorController> > & detectorControllers);

	virtual void start();
	virtual void raz();

	void refreshNumberOfDetectorChannels(int32 value);
	void updateListModeData(int index);

private:
	void resetData();

	Property<int32> dT;
	Property<int32> cleanDT;
	Property<int32> numberOfDetectorChannels;
	DynamicProperty<int32> detectorChannel;
	Property<int32> coincidenceResolution;
	Property<int32> maxBlock;

	Property<int32> totalNumberOfChannels;

	Property<int32> nOrder;
	DynamicProperty<int32> order;
	DynamicProperty<float64> globalRate;
	DynamicProperty<float64> globalCleanRate;
	DynamicProperty<float64> detectorRate;
	DynamicProperty<float64> detectorCleanRate;

	DynamicProperty<int32> crateId;
	DynamicProperty<int32> boardId;
	DynamicProperty<int32> channelId;
	DynamicProperty<float64> channelRate;

	std::auto_ptr<cameo::application::Instance> m_coincidenceApplication;
};

}

#endif
