/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef VEXP_VEXPCONTROLLER_H
#define VEXP_VEXPCONTROLLER_H

#include "controllers/tas/common/TasSettings.h"
#include "controllers/tas/common/Scattering.h"
#include "controllers/tas/common/Sample.h"
#include "controllers/tas/common/IncidentBeam.h"
#include "controllers/tas/common/TasScatteredBeam.h"
#include <boost/thread/thread.hpp>
#include <boost/thread/condition.hpp>
#include <boost/thread/mutex.hpp>
#include <memory>

namespace vexp {

class VEXPController : public ExperimentController {

public:
	//! Type of controller
	static const std::string TYPE;

	VEXPController(const std::string& name);
	VEXPController(const VEXPController& controller);
	virtual ~VEXPController();

	void updateUMatrix();
	void updateBMatrix();

	void refreshKi(float64 value);
	void refreshKf(float64 value);


	// Not used anymore.
	void refreshFloat64Property(SimpleProperty<float64>& property, float64 value);

	virtual void postConfiguration();

	Property<float64> as;
	Property<float64> bs;
	Property<float64> cs;
	Property<float64> aa;
	Property<float64> bb;
	Property<float64> cc;
	Property<float64> ax;
	Property<float64> ay;
	Property<float64> az;
	Property<float64> bx;
	Property<float64> by;
	Property<float64> bz;

	ArrayProperty<float64> u;
	ArrayProperty<float64> b;

	Property<float64> qh;
	Property<float64> qk;
	Property<float64> ql;
	Property<float64> en;
	Property<float64> qm;

	Property<float64> ki;
	Property<float64> kf;

	Property<float64> a1;
	Property<float64> a2;
	Property<float64> a3;
	Property<float64> a4;
	Property<float64> a5;
	Property<float64> a6;

	StatusProperty calculationStatus;
	Property<std::string> errorMessage;

	ControllerPtr<tas::TasSettings> tasSettings;
	ControllerPtr<tas::Sample> sample;
	ControllerPtr<tas::Scattering> scattering;
	ControllerPtr<tas::IncidentBeam> incidentBeam;
	ControllerPtr<tas::TasScatteredBeam> scatteredBeam;

	ControllerPtr<axis::AxisController> a1Controller;
	ControllerPtr<axis::AxisController> a2Controller;
	ControllerPtr<axis::AxisController> a3Controller;
	ControllerPtr<axis::AxisController> a4Controller;
	ControllerPtr<axis::AxisController> a5Controller;
	ControllerPtr<axis::AxisController> a6Controller;

protected:
	void loop();
	void calculate();

	virtual void updateError(ChangeAspect* aspect);

	std::auto_ptr<boost::thread> m_loopThread; // Not used
	boost::mutex m_mutex;
	boost::condition m_waitCondition;
	bool m_running;
	bool m_errorDuringCalculation;
};

}

#endif
