/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "RealDfk31Au03Driver.h"
#include "Dfk31Au03Driver.h"

#include <libv4l2.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/mman.h>
#include <linux/videodev2.h>
#include "png++/png.hpp"

#include "Utilities/Counter.h"

namespace dfk41au03 {

void RealDfk31Au03Driver::v4lconvert_yuyv_to_rgb24(const uint8 *src, uint8 *dest, int32 width, int32 height, int32 stride) {
	int32 j = 0;
	int32 y;

	for (y = 0; y < stride - 4; y = y + 4) {

		int32 u = src[y + 1];

		int32 v = src[y + 3];
		//            printf("src[3] %d\n",src[3]);

		int32 u1 = (((u - 128) << 7) + (u - 128)) >> 6;
		int32 rg = (((u - 128) << 1) + (u - 128) + ((v - 128) << 2) + ((v - 128) << 1)) >> 3;
		int32 v1 = (((v - 128) << 1) + (v - 128)) >> 1;

		dest[j + 0] = CLIP(src[y + 0] + v1);
		dest[j + 1] = CLIP(src[y + 0] - rg);
		dest[j + 2] = CLIP(src[y + 0] + u1);

		dest[j + 3] = CLIP(src[y + 2] + v1);
		dest[j + 4] = CLIP(src[y + 2] - rg);
		dest[j + 5] = CLIP(src[y + 2] + u1);
		j = j + 6;

		// printf("convert size : %d\n",j);
		// src += 4;

		//  src += stride - (width * 2);
	}
}

void RealDfk31Au03Driver::errno_exit(const char *s) {
	fprintf(stderr, "%s error %d, %s\n", s, errno, strerror(errno));

}

int32 RealDfk31Au03Driver::xioctl(int32 fd, int32 request, void *arg) {
	int32 r;
	do {
		r = ioctl(fd, request, arg);
	} while (-1 == r && EINTR == errno);
	return r;
}

void RealDfk31Au03Driver::process_image(const void *p_buf, const int32 len_buf) {
	v4lconvert_yuyv_to_rgb24((uint8*) p_buf, (uint8*) dest_buf, XSIZE, YSIZE, len_buf);
	owner()->RGBData = string(reinterpret_cast<const char *>(dest_buf), XSIZE * YSIZE * 3);

	uint32 p = 0;
	png::image<png::rgb_pixel> img(XSIZE, YSIZE);
	for (png::uint_32 y = 0; y < img.get_height(); ++y) {
	    for (png::uint_32 x = 0; x < img.get_width(); ++x) {
	    	img[y][x] = png::rgb_pixel(dest_buf[p], dest_buf[p + 1], dest_buf[p + 2]);
	    	p += 3;
	    }
	}
	ostringstream dec;
	img.write_stream(dec);
	int32 arraySize = dec.str().size() / 4;
	if (arraySize * 4 < dec.str().size()) {
		arraySize++;
	}
	delete [] owner()->m_pngarray;
	owner()->m_pngarray = new int32[arraySize];
	memcpy(owner()->m_pngarray, dec.str().c_str(), dec.str().size());
	owner()->PNGData.update(owner()->m_pngarray);
	owner()->PNGData.setSize(arraySize);
	owner()->PNGData.sendEvent();
}

int32 RealDfk31Au03Driver::read_frame(int32 count) {
	struct v4l2_buffer buf;

	CLEAR(buf);
	buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	buf.memory = V4L2_MEMORY_MMAP;
	if (-1 == xioctl(fd, VIDIOC_DQBUF, &buf)) {
		switch (errno) {
		case EAGAIN:
			return 0;
		case EIO:
			/* Could ignore EIO, see spec. */
			/* fall through */
		default:
			errno_exit("VIDIOC_DQBUF");
		}
	}
	assert(buf.index < n_buffers);
	if (count == 0) {
		process_image(buffers[buf.index].start, buffers[buf.index].length);
	}
	if (-1 == xioctl(fd, VIDIOC_QBUF, &buf)) {
		errno_exit("VIDIOC_QBUF");
	}

	return 1;
}

void RealDfk31Au03Driver::mainloop() {
	uint32 count;

	count = COUNT_IGNORE;
	Counter c1;
	while (count-- > 0) {
		for (;;) {
			usleep(10000);
//		fd_set fds;
//			struct timeval tv;
//			int32 r;
//			FD_ZERO(&fds);
//			FD_SET(fd, &fds);
//			/* Timeout. */
//			tv.tv_sec = 0;
//			tv.tv_usec = 100000;
//			r = select(fd, &fds, NULL, NULL, &tv);
//			if (-1 == r) {
//				if (EINTR == errno)
//					continue;
//				errno_exit("select");
//			}
//			if (0 == r) {
//				fprintf(stderr, "select timeout\n");
//			}
//			if (0 != r) {
				if (read_frame(count)) {
					break;
				}
//			}
			/* EAGAIN - continue select loop. */
		}
	}

}

void RealDfk31Au03Driver::stop_capturing() {
	enum v4l2_buf_type type;

	type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	if (-1 == xioctl(fd, VIDIOC_STREAMOFF, &type))
		errno_exit("VIDIOC_STREAMOFF");
}

void RealDfk31Au03Driver::start_capturing(void) {
	uint32 i;
	enum v4l2_buf_type type;

	for (i = 0; i < n_buffers; ++i) {
		struct v4l2_buffer buf;
		CLEAR(buf);
		buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
		buf.memory = V4L2_MEMORY_MMAP;
		buf.index = i;

		if (-1 == xioctl(fd, VIDIOC_QBUF, &buf))
			errno_exit("VIDIOC_QBUF");
	}
	type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	if (-1 == xioctl(fd, VIDIOC_STREAMON, &type))
		errno_exit("VIDIOC_STREAMON");
}

void RealDfk31Au03Driver::uninit_device() {
	uint32 i;

	for (i = 0; i < n_buffers; ++i)
		if (-1 == munmap(buffers[i].start, buffers[i].length))
			errno_exit("munmap");
	free(buffers);
}

void RealDfk31Au03Driver::init_mmap() {
	struct v4l2_requestbuffers req;

	CLEAR(req);

	req.count = 2;
	req.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	req.memory = V4L2_MEMORY_MMAP;

	if (-1 == xioctl(fd, VIDIOC_REQBUFS, &req)) {
		if (EINVAL == errno) {
			fprintf(stderr, " does not support "
					"memory mapping\n");
		} else {
			errno_exit("VIDIOC_REQBUFS");
		}
	}
	if (req.count < 2) {
		fprintf(stderr, "Insufficient buffer memory\n");
	}
	buffers = (struct buffer*) calloc(req.count, sizeof(*buffers));

	if (!buffers) {
		fprintf(stderr, "Out of memory\n");
	}

	for (n_buffers = 0; n_buffers < req.count; ++n_buffers) {
		struct v4l2_buffer buf;
		CLEAR(buf);
		buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
		buf.memory = V4L2_MEMORY_MMAP;
		buf.index = n_buffers;

		if (-1 == xioctl(fd, VIDIOC_QUERYBUF, &buf))
			errno_exit("VIDIOC_QUERYBUF");

		buffers[n_buffers].length = buf.length;
		buffers[n_buffers].start = mmap(NULL /* start anywhere */, buf.length,
		PROT_READ | PROT_WRITE /* required */,
		MAP_SHARED /* recommended */, fd, buf.m.offset);

		if (MAP_FAILED == buffers[n_buffers].start)
			errno_exit("mmap");
	}
}

void RealDfk31Au03Driver::init_device() {
	struct v4l2_capability cap;
	struct v4l2_cropcap cropcap;
	struct v4l2_crop crop;
	struct v4l2_format fmt;
	uint32 min;

	if (-1 == xioctl(fd, VIDIOC_QUERYCAP, &cap)) {
		if (EINVAL == errno) {
			fprintf(stderr, "no V4L2 device\n");
		} else {
			errno_exit("VIDIOC_QUERYCAP");
		}
	}

	stringstream myStreamString;
	myStreamString << cap.card;
	owner()->version = myStreamString.str();

	if (!(cap.capabilities & V4L2_CAP_VIDEO_CAPTURE)) {
		fprintf(stderr, "no video capture device\n");
	}
	if (!(cap.capabilities & V4L2_CAP_STREAMING)) {
		fprintf(stderr, "does not support streaming i/o\n");
	}
	/* Select video input, video standard and tune here. */
	CLEAR(cropcap);

	cropcap.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;

	if (0 == xioctl(fd, VIDIOC_CROPCAP, &cropcap)) {
		crop.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
		crop.c = cropcap.defrect; /* reset to default */

		if (-1 == xioctl(fd, VIDIOC_S_CROP, &crop)) {
			switch (errno) {
			case EINVAL:
				/* Cropping not supported. */
				break;
			default:
				/* Errors ignored. */
				break;
			}
		}
	} else {
		/* Errors ignored. */
	}
	CLEAR(fmt);

	fmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	fmt.fmt.pix.width = XSIZE;
	fmt.fmt.pix.height = YSIZE;
	fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_YUYV;
	fmt.fmt.pix.field = V4L2_FIELD_INTERLACED;

	if (-1 == xioctl(fd, VIDIOC_S_FMT, &fmt))
		errno_exit("VIDIOC_S_FMT");

	/* Note VIDIOC_S_FMT may change width and height. */

	struct v4l2_streamparm fps;
	memset(&fps, 0, sizeof(fps));
	fps.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	if (ioctl(fd, VIDIOC_G_PARM, &fps) < 0) {
		printf("Couldn't query v4l fps!\n");
	}
	if ((owner()->height() != fmt.fmt.pix.height) || (owner()->width() != fmt.fmt.pix.width) || (dest_buf == NULL)) {
		if (dest_buf != NULL) {
			delete[] dest_buf;
		}
		dest_buf = new uint8[XSIZE * YSIZE * 3];
	}
	owner()->height = YSIZE;
	owner()->width = XSIZE;
	//owner()->height=fmt.fmt.pix.pixelformat ;
	owner()->bitPerPixel = RGB24BIT;
	owner()->pixelFormat = PNG;
	fps.parm.capture.readbuffers = 0;
	fps.parm.capture.timeperframe.numerator = 2;
	fps.parm.capture.timeperframe.denominator = 15;
	if (ioctl(fd, VIDIOC_S_PARM, &fps) < 0) {
		printf("Couldn't set v4l fps!\n");
	}

	init_mmap();
}

void RealDfk31Au03Driver::close_device() {
	close(fd);
	fd = 0;
}

void RealDfk31Au03Driver::open_device() {
	string video = "/dev/video0";
	fd = open(video.c_str(), O_RDWR /* required */| O_NONBLOCK, 0);
	if (fd <= 0) {
		fprintf(stderr, "Cannot open '%s': %d, %s\n", video.c_str(), errno, strerror(errno));

	}
}

/*
 * Constructor
 */
RealDfk31Au03Driver::RealDfk31Au03Driver(Dfk31Au03Driver* owner) :
		Dfk31Au03State(owner) {
	dest_buf = NULL;
}

/*
 * Destructor
 */
RealDfk31Au03Driver::~RealDfk31Au03Driver() {
}

/*
 * init
 */
void RealDfk31Au03Driver::init() {
	delete[] dest_buf;
}

/*
 * clear
 */
void RealDfk31Au03Driver::clear() {

}

/*
 * writeParam
 */
void RealDfk31Au03Driver::writeParam() {

}

/*
 * synchroniseRead
 */
void RealDfk31Au03Driver::synchroniseRead() {

	open_device();
	init_device();
	start_capturing();
	mainloop();
	stop_capturing();
	uninit_device();
	close_device();
	/*
	 * Calculate the detector sum here
	 */

	owner()->m_SynchroniseReadTerminated = true;
}

/*
 * read
 */
void RealDfk31Au03Driver::read() {
}

/*
 * start
 */
void RealDfk31Au03Driver::start() {
}

/*
 * resume
 */
void RealDfk31Au03Driver::resume() {
}

/*
 * pause
 */
void RealDfk31Au03Driver::pause() {
}

/*
 * stop
 */
void RealDfk31Au03Driver::stop() {
}

/*
 * readStatus
 */
void RealDfk31Au03Driver::readStatus() {

	if (owner()->m_SynchroniseReadActivated == true) {
		if (owner()->m_SynchroniseReadTerminated == true) {
			owner()->m_SynchroniseReadActivated = false;
			owner()->m_SynchroniseReadTerminated = false;
			owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
		}
	} else {
		owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	}

	// Status command
	owner()->computeAcquisitionStatus(0);
	owner()->computeDetectorStatus(0);
}

/*
 * readInfos
 */
void RealDfk31Au03Driver::readInfos() {
}

}
