/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "StereoFluxgateDef.h"
#include "StereoFluxgateState.h"
#include "StereoFluxgateDriver.h"
#include "PerfectStereoFluxgateDriver.h"
#include "RealStereoFluxgateDriver.h"
#include "drivers/global/DriversCommands.h"

namespace stereo_fluxgate {

const std::string StereoFluxgateDriver::TYPE = "stereo_fluxgate";

/*
 * Constructor
 */
StereoFluxgateDriver::StereoFluxgateDriver(const string& name) : DeviceDriver(name){

	registerStates(new RealStereoFluxgateDriver(this), new PerfectStereoFluxgateDriver(this), new PerfectStereoFluxgateDriver(this));

	nbfluxgate.init(this, SAVE, "nbfluxgate");
	Bx.init(this, NOSAVE, "Bx");
	By.init(this, NOSAVE, "By");
	Bz.init(this, NOSAVE, "Bz");
	ichannel.init(this, NOSAVE, "ichannel");

	isconnected.init(this, NOSAVE, "isconnected");
	isconfigured.init(this, NOSAVE, "isconfigured");
	// Init functions
	registerFunction(STEREOFLUXGATE_FUNCTION);
	registerRefresher(nbfluxgate,&StereoFluxgateDriver::refreshNbFluxgateProperty, this);
	/*
	* Register the Spy and Observer commands necessary to do the updates.
	*/
	registerSpyCommand(driver::READ_COMMAND, 1);//5 sec
	registerObserverCommand(driver::READ_COMMAND, 2);//20 msec

	isconnected=false;
	isconfigured=false;
}


/*
 * Destructor
 */
StereoFluxgateDriver::~StereoFluxgateDriver() {
	// Empty
}


/*
 * execute
 */
void StereoFluxgateDriver::execute(const string& aCommand) {

	if ((aCommand != driver::STOP_COMMAND) && (aCommand != driver::STATUS_COMMAND) && (aCommand != driver::READ_COMMAND)) {
		commandProgression = PROGRESSION_UNKNOWNSTATE_DEVICE_CONTAINER;
	}
	StereoFluxgateState* currentState = dynamic_cast<StereoFluxgateState*>(getCurrentState());

	// Check command
	if (aCommand == driver::INIT_COMMAND) {
		currentState->init();
	} else if (aCommand == driver::STATUS_COMMAND) {
		// Status command
	//	cout << "Status command" << endl;

	} else if (aCommand == driver::READ_COMMAND) {
		// Read command
		//cout << "READ COMMAND configured=" << isconfigured()<< endl;
		if(isconfigured()) currentState->readFieldValues();
	}

}

void StereoFluxgateDriver::refreshNbFluxgateProperty(int32 value) throw (CannotSetValue) {
	Bx.resize(value);
	By.resize(value);
	Bz.resize(value);
	ichannel.resize(value);
	for(int i=0;i<value;i++){
		Bx.set(i,0);
		By.set(i,0);
		Bz.set(i,0);
		ichannel.set(i,i);
	}
}

}
